import 'dart:async';
import 'dart:typed_data';
import 'package:libcli/log/log.dart' as log;
import 'package:protobuf/protobuf.dart' as $pb;
import 'package:http/http.dart' as http;
import 'command_protobuf.dart';
import 'command_url.dart';
import 'command_http.dart';
import 'package:libcli/contract/contract.dart' as c;
import 'package:libcli/command/commands/shared/err.pb.dart';

const _here = 'command';

abstract class ProtoObject extends $pb.GeneratedMessage {
  int mapIdXXX();
}

/// communicate with server with command using ajax,protobuf and command pattern
/// it basically send a action command and receive response command
abstract class Service {
  /// remote service url
  String url;

  /// timeout define request timeout in ms
  final int timeout;

  /// slow define slow network in ms
  final int slow;

  Function onError;

  /// find object by id
  ProtoObject newObjectByID(int id, List<int> bytes);

  /// Service create service with remote cloud function name,timeout and slow
  ///
  /// debug port used in debug branch
  Service(String funcName, int debugPort, this.timeout, this.slow) {
    url = serviceUrl(funcName, debugPort);
  }

  /// send command and return result, return null when something wrong
  ///

  ///     EchoAction echoAction = new EchoAction();
  ///     echoAction.text = 'helloWorld';
  ///     service.send(echoAction).then((result) {
  ///       if(result != null){};
  ///     });
  Future<dynamic> send(ProtoObject obj) async {
    assert(url != null && url.length > 0);
    http.Client client = http.Client();
    return sendWithClient(client, obj);
  }

  /// sendWithClient send command using client
  ///
  ///     StringResponse response = await service.sendWithClient(client, action);
  ///     expect(response.text, 'hi');
  Future<dynamic> sendWithClient(http.Client client, ProtoObject obj) async {
    try {
      log.debug(_here, 'send ${obj.runtimeType} to $url');
      Uint8List bytes = encode(obj);
      List<int> ret = await post(client, url, bytes, timeout, slow, onError);
      if (ret != null) {
        ProtoObject retObj = decode(ret, this);
        if (retObj is Err && retObj.code == 0) {
          log.debug(_here, 'receive ok from $url');
          return true;
        }
        log.debug(_here, 'receive ${retObj.runtimeType} from $url');
        return retObj;
      }
    } catch (e, s) {
      var errId = log.error(_here, e, s);
      return giveup(c.EError(errId));
    }
    log.debugWarning(_here, 'receive null from $url');
    return null;
  }
}
